@@ -386,7 +386,7 @@ AdjustClocksToTransactionHighest(List *nodeConnectionList,
 
 	/* Set the clock value on participating worker nodes */
 	appendStringInfo(queryToSend,
-					 "SELECT pg_catalog.citus_internal_adjust_local_clock_to_remote"
+					 "SELECT citus_internal.adjust_local_clock_to_remote"
 					 "('(%lu, %u)'::pg_catalog.cluster_clock);",
 					 transactionClockValue->logical, transactionClockValue->counter);
 
@@ -420,6 +420,11 @@ PrepareAndSetTransactionClock(void)
 		MultiConnection *connection = dlist_container(MultiConnection, transactionNode,
 													  iter.cur);
 		WorkerNode *workerNode = FindWorkerNode(connection->hostname, connection->port);
+		if (!workerNode)
+		{
+			ereport(WARNING, errmsg("Worker node is missing"));
+			continue;
+		}
 
 		/* Skip the node if we already in the list */
 		if (list_member_int(nodeList, workerNode->groupId))
