@@ -173,7 +173,6 @@ BuildCreatePublicationStmt(Oid publicationId)
 												PUBLICATION_PART_ROOT :
 												PUBLICATION_PART_LEAF);
 	Oid relationId = InvalidOid;
-	int citusTableCount PG_USED_FOR_ASSERTS_ONLY = 0;
 
 	/* mainly for consistent ordering in test output */
 	relationIds = SortList(relationIds, CompareOids);
@@ -187,11 +186,6 @@ BuildCreatePublicationStmt(Oid publicationId)
 			BuildPublicationRelationObjSpec(relationId, publicationId, tableOnly);
 
 		createPubStmt->pubobjects = lappend(createPubStmt->pubobjects, publicationObject);
-
-		if (IsCitusTable(relationId))
-		{
-			citusTableCount++;
-		}
 	}
 
 	/* WITH (publish_via_partition_root = true) option */
