@@ -62,7 +62,7 @@ TaskListRequiresRollback(List *taskList)
 	}
 
 	Task *task = (Task *) linitial(taskList);
-	if (task->cannotBeExecutedInTransction)
+	if (task->cannotBeExecutedInTransaction)
 	{
 		/* vacuum, create index concurrently etc. */
 		return false;
@@ -165,7 +165,7 @@ TaskListCannotBeExecutedInTransaction(List *taskList)
 	Task *task = NULL;
 	foreach_declared_ptr(task, taskList)
 	{
-		if (task->cannotBeExecutedInTransction)
+		if (task->cannotBeExecutedInTransaction)
 		{
 			return true;
 		}
