@@ -235,7 +235,20 @@ CitusExecutorRun(QueryDesc *queryDesc,
 			/* postgres will switch here again and will restore back on its own */
 			MemoryContextSwitchTo(oldcontext);
 
-			standard_ExecutorRun(queryDesc, direction, count, execute_once);
+			#if PG_VERSION_NUM >= PG_VERSION_18
+
+			/* PG18+ drops the “execute_once” argument */
+			standard_ExecutorRun(queryDesc,
+								 direction,
+								 count);
+		#else
+
+			/* PG17-: original four-arg signature */
+			standard_ExecutorRun(queryDesc,
+								 direction,
+								 count,
+								 execute_once);
+		#endif
 		}
 
 		if (totalTime)
@@ -675,7 +688,7 @@ ExecuteQueryIntoDestReceiver(Query *query, ParamListInfo params, DestReceiver *d
  * ExecutePlanIntoDestReceiver executes a query plan and sends results to the given
  * DestReceiver.
  */
-void
+uint64
 ExecutePlanIntoDestReceiver(PlannedStmt *queryPlan, ParamListInfo params,
 							DestReceiver *dest)
 {
@@ -688,16 +701,44 @@ ExecutePlanIntoDestReceiver(PlannedStmt *queryPlan, ParamListInfo params,
 	/* don't display the portal in pg_cursors, it is for internal use only */
 	portal->visible = false;
 
-	PortalDefineQuery(portal,
-					  NULL,
-					  "",
-					  CMDTAG_SELECT,
-					  list_make1(queryPlan),
-					  NULL);
+	PortalDefineQuery(
+		portal,
+		NULL,                 /* no prepared statement name */
+		"",                   /* query text */
+		CMDTAG_SELECT,        /* command tag */
+		list_make1(queryPlan),/* list of PlannedStmt* */
+		NULL                  /* no CachedPlan */
+		);
 
 	PortalStart(portal, params, eflags, GetActiveSnapshot());
-	PortalRun(portal, count, false, true, dest, dest, NULL);
+
+
+	QueryCompletion qc = { 0 };
+
+#if PG_VERSION_NUM >= PG_VERSION_18
+
+/* PG 18+: six-arg signature (drop the run_once bool) */
+	PortalRun(portal,
+			  count,  /* how many rows to fetch */
+			  false,  /* isTopLevel */
+			  dest,   /* DestReceiver *dest */
+			  dest,   /* DestReceiver *altdest */
+			  &qc);  /* QueryCompletion *qc */
+#else
+
+/* PG 17-: original seven-arg signature */
+	PortalRun(portal,
+			  count,  /* how many rows to fetch */
+			  false,  /* isTopLevel */
+			  true,   /* run_once */
+			  dest,   /* DestReceiver *dest */
+			  dest,   /* DestReceiver *altdest */
+			  &qc);  /* QueryCompletion *qc */
+#endif
+
 	PortalDrop(portal, false);
+
+	return qc.nprocessed;
 }
 
 
