@@ -746,7 +746,12 @@ GetRelationIdentityOrPK(Relation rel)
 
 	if (!OidIsValid(idxoid))
 	{
+/* Determine the index OID of the primary key (PG18 adds a second parameter) */
+#if PG_VERSION_NUM >= PG_VERSION_18
+		idxoid = RelationGetPrimaryKeyIndex(rel, false /* deferred_ok */);
+#else
 		idxoid = RelationGetPrimaryKeyIndex(rel);
+#endif
 	}
 
 	return idxoid;
