@@ -209,7 +209,9 @@ END;
     -- this might take long depending on the number of tables & objects ...
     RAISE NOTICE 'Preparing to sync the metadata to all nodes';
 
-    PERFORM start_metadata_sync_to_all_nodes();
+    PERFORM start_metadata_sync_to_node(nodename,nodeport)
+    FROM
+      pg_dist_node WHERE groupid != 0 AND noderole = 'primary';
   END;
 
   RETURN true;
