@@ -11,9 +11,7 @@ CREATE OR REPLACE FUNCTION pg_catalog.get_rebalance_progress()
                 targetport int,
                 progress bigint,
                 source_shard_size bigint,
-                target_shard_size bigint,
-                operation_type text
-            )
+                target_shard_size bigint)
   AS 'MODULE_PATHNAME'
   LANGUAGE C STRICT;
 COMMENT ON FUNCTION pg_catalog.get_rebalance_progress()
