@@ -324,6 +324,16 @@ GetRangeTblKind(RangeTblEntry *rte)
 			break;
 		}
 
+		#if PG_VERSION_NUM >= PG_VERSION_18
+
+		/* new in PG18: GROUP RTE, just map it straight through */
+		case RTE_GROUP:
+		{
+			rteKind = (CitusRTEKind) rte->rtekind;
+			break;
+		}
+
+		#endif
 		case RTE_FUNCTION:
 		{
 			/*
