@@ -343,8 +343,10 @@ extern void LookupTaskPlacementHostAndPort(ShardPlacement *taskPlacement, char *
 										   int *nodePort);
 extern bool IsDummyPlacement(ShardPlacement *taskPlacement);
 extern StringInfo GenerateSizeQueryOnMultiplePlacements(List *shardIntervalList,
+														Oid indexId,
 														SizeQueryType sizeQueryType,
-														bool optimizePartitionCalculations);
+														bool optimizePartitionCalculations
+														);
 extern List * RemoveCoordinatorPlacementIfNotSingleNode(List *placementList);
 
 /* Function declarations to modify shard and shard placement data */
@@ -385,6 +387,7 @@ extern void EnsureUndistributeTenantTableSafe(Oid relationId, const char *operat
 extern TableConversionReturn * UndistributeTable(TableConversionParameters *params);
 extern void UndistributeTables(List *relationIdList);
 
+extern void EnsureObjectAndDependenciesExistOnAllNodes(const ObjectAddress *target);
 extern void EnsureAllObjectDependenciesExistOnAllNodes(const List *targets);
 extern DeferredErrorMessage * DeferErrorIfCircularDependencyExists(const
 																   ObjectAddress *
@@ -398,7 +401,7 @@ extern bool ShouldPropagateAnyObject(List *addresses);
 /* Remaining metadata utility functions  */
 extern Oid TableOwnerOid(Oid relationId);
 extern char * TableOwner(Oid relationId);
-extern void EnsureTablePermissions(Oid relationId, AclMode mode);
+extern void EnsureTablePermissions(Oid relationId, AclMode mode, AclMaskHow mask);
 extern void EnsureTableOwner(Oid relationId);
 extern void EnsureHashDistributedTable(Oid relationId);
 extern void EnsureHashOrSingleShardDistributedTable(Oid relationId);
@@ -464,4 +467,9 @@ extern bool IsBackgroundJobStatusTerminal(BackgroundJobStatus status);
 extern bool IsBackgroundTaskStatusTerminal(BackgroundTaskStatus status);
 extern Oid BackgroundJobStatusOid(BackgroundJobStatus status);
 extern Oid BackgroundTaskStatusOid(BackgroundTaskStatus status);
+extern int GetAutoConvertedAttrIndexInPgDistPartition(TupleDesc tupleDEsc);
+
+/* from node_metadata.c */
+extern void LockShardsInWorkerPlacementList(WorkerNode *workerNode, LOCKMODE lockMode);
+extern void ActivateCloneNodeAsPrimary(WorkerNode *workerNode);
 #endif   /* METADATA_UTILITY_H */
