@@ -265,46 +265,6 @@ ROLLBACK;
 -- make sure that we read the correct value for "b" when doing index only scan
 SELECT b=980 FROM include_test WHERE a = 980;
 
--- some tests with distributed & partitioned tables --
-
-CREATE TABLE dist_part_table(
-  dist_col INT,
-  part_col TIMESTAMPTZ,
-  col1 TEXT
-) PARTITION BY RANGE (part_col);
-
--- create an index before creating a columnar partition
-CREATE INDEX dist_part_table_btree ON dist_part_table (col1);
-
--- columnar partition
-CREATE TABLE p0 PARTITION OF dist_part_table
-FOR VALUES FROM ('2020-01-01') TO ('2020-02-01')
-USING columnar;
-
-SELECT create_distributed_table('dist_part_table', 'dist_col');
-
--- columnar partition
-CREATE TABLE p1 PARTITION OF dist_part_table
-FOR VALUES FROM ('2020-02-01') TO ('2020-03-01')
-USING columnar;
-
--- row partition
-CREATE TABLE p2 PARTITION OF dist_part_table
-FOR VALUES FROM ('2020-03-01') TO ('2020-04-01');
-
-INSERT INTO dist_part_table VALUES (1, '2020-03-15', 'str1', POINT(1, 1));
-
--- insert into columnar partitions
-INSERT INTO dist_part_table VALUES (1, '2020-01-15', 'str2', POINT(2, 2));
-INSERT INTO dist_part_table VALUES (1, '2020-02-15', 'str3', POINT(3, 3));
-
--- create another index after creating a columnar partition
-CREATE UNIQUE INDEX dist_part_table_unique ON dist_part_table (dist_col, part_col);
-
--- verify that indexes are created on columnar partitions
-SELECT COUNT(*)=2 FROM pg_indexes WHERE tablename = 'p0';
-SELECT COUNT(*)=2 FROM pg_indexes WHERE tablename = 'p1';
-
 -- unsupported index types --
 
 -- gin --
