@@ -5,16 +5,16 @@
 -- Tests to verify that we correctly prune unreferenced shards. For this, we
 -- need to increase the logging verbosity of messages displayed on the client.
 
-SET citus.explain_distributed_queries TO off;
-SET client_min_messages TO DEBUG2;
+
+SET citus.next_shard_id TO 770000;
 
 -- Adding additional l_orderkey = 1 to make this query not router executable
-SELECT l_orderkey, l_linenumber, l_shipdate FROM lineitem WHERE l_orderkey = 9030 or l_orderkey = 1;
+SELECT l_orderkey, l_linenumber, l_shipdate FROM lineitem WHERE l_orderkey = 9030 or l_orderkey = 1 ORDER BY 1,2;
 
 -- We use the l_linenumber field for the following aggregations. We need to use
 -- an integer type, as aggregations on numerics or big integers return numerics
 -- of unknown length. When the numerics are read into our temporary table, they
--- trigger the the creation of toasted tables and indexes. This in turn prints
+-- trigger the creation of toasted tables and indexes. This in turn prints
 -- non-deterministic debug messages. To avoid this chain, we use l_linenumber.
 
 SELECT sum(l_linenumber), avg(l_linenumber) FROM lineitem WHERE l_orderkey > 9030;
@@ -38,7 +38,7 @@ CREATE TABLE varchar_partitioned_table
 (
 	varchar_column varchar(100)
 );
-SELECT master_create_distributed_table('varchar_partitioned_table', 'varchar_column', 'append');
+SELECT create_distributed_table('varchar_partitioned_table', 'varchar_column', 'range');
 
 -- Create logical shards and shard placements with shardid 100,101
 
@@ -67,7 +67,7 @@ CREATE TABLE array_partitioned_table
 (
 	array_column text[]
 );
-SELECT master_create_distributed_table('array_partitioned_table', 'array_column', 'append');
+SELECT create_distributed_table('array_partitioned_table', 'array_column', 'range');
 SET client_min_messages TO DEBUG2;
 
 -- Create logical shard with shardid 102, 103
@@ -92,6 +92,7 @@ INSERT INTO pg_dist_shard_placement (shardid, shardstate, shardlength, nodename,
 	LIMIT 1;
 
 -- Create composite type partitioned table
+RESET client_min_messages; -- avoid debug messages
 
 CREATE TYPE composite_type AS
 (
@@ -100,12 +101,11 @@ CREATE TYPE composite_type AS
 	varchar_column varchar(50)
 );
 
-RESET client_min_messages; -- avoid debug messages about toast index creation
 CREATE TABLE composite_partitioned_table
 (
 	composite_column composite_type
 );
-SELECT master_create_distributed_table('composite_partitioned_table', 'composite_column', 'append');
+SELECT create_distributed_table('composite_partitioned_table', 'composite_column', 'range');
 SET client_min_messages TO DEBUG2;
 
 -- Create logical shard with shardid 104, 105
@@ -127,15 +127,21 @@ INSERT INTO pg_dist_shard_placement (shardid, shardstate, shardlength, nodename,
 	ORDER BY nodename, nodeport ASC
 	LIMIT 1;
 
--- Verify that shard pruning works. Note that these queries should all prune
--- one shard.
+-- Verify that shard pruning works. Note that these queries should all
+-- prune one shard (see task count).  As these tables don't exist
+-- remotely, temporarily disable WARNING messages.
+SET client_min_messages TO ERROR;
 
-EXPLAIN SELECT count(*) FROM varchar_partitioned_table WHERE varchar_column = 'BA2';
 
-EXPLAIN SELECT count(*) FROM array_partitioned_table
+EXPLAIN (COSTS OFF)
+SELECT count(*) FROM varchar_partitioned_table WHERE varchar_column = 'BA2';
+
+EXPLAIN (COSTS OFF)
+SELECT count(*) FROM array_partitioned_table
 	WHERE array_column > '{BA1000U2AMO4ZGX, BZZXSP27F21T6}';
 
-EXPLAIN SELECT count(*) FROM composite_partitioned_table
+EXPLAIN (COSTS OFF)
+SELECT count(*) FROM composite_partitioned_table
 	WHERE composite_column < '(b,5,c)'::composite_type;
 
 SET client_min_messages TO NOTICE;
