@@ -551,6 +551,123 @@ BEGIN
 END;
 $func$ LANGUAGE plpgsql;
 
+-- Returns pg_seclabels entries from all nodes in the cluster for which
+-- the object name is the input.
+CREATE OR REPLACE FUNCTION get_citus_tests_label_provider_labels(object_name text,
+                                                                 master_port INTEGER DEFAULT 57636,
+                                                                 worker_1_port INTEGER DEFAULT 57637,
+                                                                 worker_2_port INTEGER DEFAULT 57638)
+RETURNS TABLE (
+    node_type text,
+    result text
+)
+AS $func$
+DECLARE
+    pg_seclabels_cmd TEXT := 'SELECT to_jsonb(q.*) FROM (' ||
+                             'SELECT provider, objtype, label FROM pg_seclabels ' ||
+                             'WHERE objname = ''' || object_name || ''') q';
+BEGIN
+    RETURN QUERY
+    SELECT
+        CASE
+            WHEN nodeport = master_port THEN 'coordinator'
+            WHEN nodeport = worker_1_port THEN 'worker_1'
+            WHEN nodeport = worker_2_port THEN 'worker_2'
+            ELSE 'unexpected_node'
+        END AS node_type,
+        a.result
+    FROM run_command_on_all_nodes(pg_seclabels_cmd) a
+    JOIN pg_dist_node USING (nodeid)
+    ORDER BY node_type;
+END;
+$func$ LANGUAGE plpgsql;
+
+-- For all nodes, returns database properties of given database, except
+-- oid, datfrozenxid and datminmxid.
+--
+-- Also returns whether the node has a pg_dist_object record for the database
+-- and whether there are any stale pg_dist_object records for a database.
+CREATE OR REPLACE FUNCTION check_database_on_all_nodes(p_database_name text)
+RETURNS TABLE (node_type text, result text)
+AS $func$
+DECLARE
+  pg_ge_15_17_options text := '';
+  pg_ge_16_options text := '';
+BEGIN
+  IF EXISTS (SELECT 1 FROM pg_attribute WHERE attrelid = 'pg_database'::regclass AND attname = 'daticulocale') THEN
+    pg_ge_15_17_options := ', daticulocale, datcollversion, datlocprovider';
+  ELSIF EXISTS (SELECT 1 FROM pg_attribute WHERE attrelid = 'pg_database'::regclass AND attname = 'datlocale') THEN
+    pg_ge_15_17_options := ', datlocale as daticulocale, datcollversion, datlocprovider';
+  ELSE
+    pg_ge_15_17_options := $$, null as daticulocale, null as datcollversion, 'c' as datlocprovider$$;
+  END IF;
+
+  IF EXISTS (SELECT 1 FROM pg_attribute WHERE attrelid = 'pg_database'::regclass AND attname = 'daticurules') THEN
+    pg_ge_16_options := ', daticurules';
+  ELSE
+    pg_ge_16_options := ', null as daticurules';
+  END IF;
+
+  RETURN QUERY
+    SELECT
+      CASE WHEN (groupid = 0 AND groupid = (SELECT groupid FROM pg_dist_local_group)) THEN 'coordinator (local)'
+           WHEN (groupid = 0) THEN 'coordinator (remote)'
+           WHEN (groupid = (SELECT groupid FROM pg_dist_local_group)) THEN 'worker node (local)'
+           ELSE 'worker node (remote)'
+      END AS node_type,
+      q2.result
+    FROM run_command_on_all_nodes(
+        format(
+            $$
+            SELECT to_jsonb(q.*)
+            FROM (
+                SELECT
+                (
+                    SELECT to_jsonb(database_properties.*)
+                    FROM (
+                        SELECT datname, pa.rolname as database_owner,
+                            pg_encoding_to_char(pd.encoding) as encoding,
+                            datistemplate, datallowconn, datconnlimit, datacl,
+                            pt.spcname AS tablespace, datcollate, datctype
+                            %2$s -- >= pg15 & pg17 options
+                            %3$s -- >= pg16 options
+                        FROM pg_database pd
+                        JOIN pg_authid pa ON pd.datdba = pa.oid
+                        JOIN pg_tablespace pt ON pd.dattablespace = pt.oid
+                        WHERE datname = '%1$s'
+                    ) database_properties
+                ) AS database_properties,
+                (
+                    SELECT COUNT(*)=1
+                    FROM pg_dist_object WHERE objid = (SELECT oid FROM pg_database WHERE datname = '%1$s')
+                ) AS pg_dist_object_record_for_db_exists,
+                (
+                    SELECT COUNT(*) > 0
+                    FROM pg_dist_object
+                    WHERE classid = 1262 AND objid NOT IN (SELECT oid FROM pg_database)
+                ) AS stale_pg_dist_object_record_for_a_db_exists
+            ) q
+            $$,
+            p_database_name, pg_ge_15_17_options, pg_ge_16_options
+        )
+    ) q2
+    JOIN pg_dist_node USING (nodeid);
+END;
+$func$ LANGUAGE plpgsql;
+
+CREATE OR REPLACE FUNCTION check_database_privileges(role_name text, db_name text, permissions text[])
+RETURNS TABLE(permission text, result text)
+AS $func$
+DECLARE
+    permission text;
+BEGIN
+    FOREACH permission IN ARRAY permissions
+    LOOP
+        RETURN QUERY EXECUTE format($inner$SELECT %s, result FROM run_command_on_all_nodes($$select has_database_privilege(%s,%s,%s); $$)$inner$,
+        quote_literal(permission), quote_literal(role_name), quote_literal(db_name), quote_literal(permission));
+    END LOOP;
+END;
+$func$ LANGUAGE plpgsql;
 CREATE or REPLACE FUNCTION initplan_references_to_pg17(text) returns text AS $$
 DECLARE
   expr_parts text[];
@@ -660,33 +777,3 @@ begin
     end loop;
 end;
 $$;
--- Returns pg_seclabels entries from all nodes in the cluster for which
--- the object name is the input.
-CREATE OR REPLACE FUNCTION get_citus_tests_label_provider_labels(object_name text,
-                                                                 master_port INTEGER DEFAULT 57636,
-                                                                 worker_1_port INTEGER DEFAULT 57637,
-                                                                 worker_2_port INTEGER DEFAULT 57638)
-RETURNS TABLE (
-    node_type text,
-    result text
-)
-AS $func$
-DECLARE
-    pg_seclabels_cmd TEXT := 'SELECT to_jsonb(q.*) FROM (' ||
-                             'SELECT provider, objtype, label FROM pg_seclabels ' ||
-                             'WHERE objname = ''' || object_name || ''') q';
-BEGIN
-    RETURN QUERY
-    SELECT
-        CASE
-            WHEN nodeport = master_port THEN 'coordinator'
-            WHEN nodeport = worker_1_port THEN 'worker_1'
-            WHEN nodeport = worker_2_port THEN 'worker_2'
-            ELSE 'unexpected_node'
-        END AS node_type,
-        a.result
-    FROM run_command_on_all_nodes(pg_seclabels_cmd) a
-    JOIN pg_dist_node USING (nodeid)
-    ORDER BY node_type;
-END;
-$func$ LANGUAGE plpgsql;
