@@ -2,9 +2,6 @@
 -- MULTI_TPCH_QUERY14
 --
 
--- Change configuration to treat lineitem and orders tables as large
-
-SET citus.large_table_shard_count TO 2;
 
 -- Query #14 from the TPC-H decision support benchmark
 
